<?php
/*
 Plugin Name: WP e-Commerce Bring Fraktguide
 Plugin URI: http://wordpress.org/extend/plugins/wp-e-commerce-bring-fraktguide/
 Description: Bring Fraktguide Shipping Module for WP e-Commerce. Uses XML API at http://fraktguide.bring.no/fraktguide/xmlOverHttp.do
 Version: 0.3
 Author: Leif-Arne Helland
 Author URI: http://www.hwu.no/
*/
/*
 This program is free software; you can redistribute it and/or modify it under the terms of the GNU General Public License as published by the Free Software Foundation, version 2.
 This program is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 */
class bring_fraktguide {
	var $internal_name;
	var $name;
	var $requires_weight;
	var $needs_zipcode;
	var $is_external;
	var $services = array();
	var $base_zipcode;
	var $base_country;
	var $options;
	var $use_cod;
	var $output;
	function bring_fraktguide () {
		// An internal reference to the method - must be unique!
		$this->internal_name = "bring_fraktguide";
		// $this->name is how the method will appear to end users
		$this->name = __('Bring Shipping Guide', 'bfg');
		$this->requires_weight = true;
		$this->needs_zipcode = true;
		// Set to FALSE - doesn't really do anything :)
		$this->is_external = true;
		// Initialise the list of available postage services
		$this->services['MINIPAKKE'] = __('Mini Package', 'bfg');
		$this->services['A-POST'] = __('Letter (A-Priority)', 'bfg');
		$this->services['B-POST'] = __('Letter (B-Economy)', 'bfg');
		$this->services['NORGESPAKKE'] = __('At the post office or post in shop (Norway Package)', 'bfg');
		$this->services['BPAKKE_DOR-DOR'] = __('Daytime, at work 0800-1600', 'bfg');
		$this->services['SERVICEPAKKE'] = __('At the post office or post in shop', 'bfg');
		$this->services['PA_DOREN'] = __('Evening, at home 1700-2100', 'bfg');
		$this->services['EKSPRESS09'] = __('Express by 0900', 'bfg');
		$this->services['EKSPRESS07'] = __('Express by 0700', 'bfg');
		// Attempt to load the existing options
		$this->options = get_option($this->getInternalName().'_options');
		// Get country and zipcode
		$this->base_country = get_option('base_country');
		$this->base_zipcode = get_option('base_zipcode');
		$this->output = "";
		
		if (!$this->options) {
			// Initialise the options
			$this->options = array();
			foreach ($this->services as $code => $value) {
				$this->options['services'][$code] = true;
			}
		}
		
		return true;
	
	}
	
	function bring_fraktguide_install() {
		$new_options = array(
			'services' => array(
				'MINIPAKKE' => true,
				'A-POST' => true,
				'B-POST' => true,			
				'NORGESPAKKE' => true,
				'BPAKKE_DOR-DOR' => true,
				'SERVICEPAKKE' => true,
				'PA_DOREN' => true,
				'EKSPRESS09' => true,
				'EKSPRESS07' => true
			),
			'use_cod' => false, 		// true or false, defaults to false
			'quote_method' => 'total', 	// total or items, defaults to total
			'price_vat' => 'without'	// with or without, defaults to without
		);
		add_option( 'bring_fraktguide_options', $new_options );
	}
	
	/* You must always supply this */
	function getName() {
		return $this->name;
	}
	
	/* You must always supply this */
	function getInternalName() {
		return $this->internal_name;
	}
	
	
	/* Use this function to return HTML for setting any configuration options for your shipping method
	 * This will appear in the WP E-Commerce admin area under Products > Settings > Shipping
         *
	 * Whatever you output here will be wrapped inside the right <form> tags, and also
	 * a <table> </table> block */
	function getForm() {
		// Only for Norwegian merchants
		if ($this->base_country != 'NO') {
			return __('This shipping module only works if the base country in settings, region is set to Norway.', 'bfg');
		}
		
		// Base postcode must be set
		if (strlen($this->base_zipcode) != 4) {
			return __('You must set your base postcode above before this shipping module will work.', 'bfg');
		}
		$options = get_option($this->getInternalName().'_options');
		
		$this->output .= '<tr>';
		$this->output .= '	<td>';
		$this->output .= '		<p><strong>'.__('Select available services from Bring Shipping Guide', 'bfg').'</strong></p>';
		foreach ($this->services as $code => $value) {
			$checked = $this->options['services'][$code] ? "checked='checked'" : '';
			$this->output .= "		<label class=\"selectit\"><input type='checkbox' {$checked} name='{$this->getInternalName()}_options[services][{$code}]' />{$this->services[$code]}</label><br />\n\r";
		}
		$this->output .= '	</td>';
		$this->output .= '</tr>';		
		$this->output .= '<tr>';
		$this->output .= '	<td>';
		$this->output .= '		<p><strong>'.__('Settings', 'bfg').'</strong></p>';
		$checked = $this->options['use_cod'] ? "checked='checked'" : '';
		$this->output .= "		<label class=\"selectit\"><input type='checkbox' {$checked} name='{$this->getInternalName()}_options[use_cod]' />".__('Use Collect on Delivery', 'bfg')."</label><br />\n\r";
		$this->output .= '<br />';
		if (!isset($this->options['quote_method'])) {
			$this->options['quote_method'] = 'total';
		}
		$this->output .= '		<label title="'.__('Get price for entire cart as single package', 'bfg').'"><input type="radio" name="'.$this->getInternalName().'_options[quote_method]" value="total" '.($this->options['quote_method'] == 'total' ? 'checked' : '').'>'.__('Get price for entire cart as single package', 'bfg').'</label><br />';
		$this->output .= '		<label title="'.__('Get one price per item and sum up total', 'bfg').'"><input type="radio" name="'.$this->getInternalName().'_options[quote_method]" value="items" '.($this->options['quote_method'] == 'items' ? 'checked' : '').'>'.__('Get one price per item and sum up total', 'bfg').'</label><br />';		
		$this->output .= '<br />';
		if (!isset($this->options['price_vat'])) {
			$this->options['price_vat'] = 'with';
		}
		$this->output .= '		<label title="'.__('Get price with VAT', 'bfg').'"><input type="radio" name="'.$this->getInternalName().'_options[price_vat]" value="with" '.($this->options['price_vat'] == 'with' ? 'checked' : '').'>'.__('Get price with VAT', 'bfg').'</label><br />';
		$this->output .= '		<label title="'.__('Get price without VAT', 'bfg').'"><input type="radio" name="'.$this->getInternalName().'_options[price_vat]" value="without" '.($this->options['price_vat'] == 'without' ? 'checked' : '').'>'.__('Get price without VAT', 'bfg').'</label>';		
		$this->output .= "<input type='hidden' name='{$this->internal_name}_updateoptions' value='true'>";
		
		return $this->output;
	}
	
	/* Use this function to store the settings submitted by the form above
	 * Submitted form data is in $_POST */
	function submit_form() {

		$this->options = array();
		
		// Only continue if this module's options were updated
		if ( !isset($_POST["{$this->internal_name}_updateoptions"]) || !$_POST["{$this->internal_name}_updateoptions"] ) return;
		
		if (isset($_POST[$this->getInternalName().'_options'])) {
			if (isset($_POST[$this->getInternalName().'_options']['services'])) {
				foreach ($this->services as $code => $name) {
					$this->options['services'][$code] = isset($_POST[$this->getInternalName().'_options']['services'][$code]) ? true : false;
				}
			}
			if (isset($_POST[$this->getInternalName().'_options']['use_cod'])) {
				$this->options['use_cod'] = isset($_POST[$this->getInternalName().'_options']['use_cod']) ? true : false;
			}
			if (isset($_POST[$this->getInternalName().'_options']['quote_method'])) {
				$this->options['quote_method'] = $_POST[$this->getInternalName().'_options']['quote_method'];
			}
			if (isset($_POST[$this->getInternalName().'_options']['price_vat'])) {
				$this->options['price_vat'] = $_POST[$this->getInternalName().'_options']['price_vat'];
			}
		}
		update_option($this->getInternalName().'_options', $this->options);
		return true;
	}
	
	/* If there is a per-item shipping charge that applies irrespective of the chosen shipping method
         * then it should be calculated and returned here. The value returned from this function is used
         * as-is on the product pages. It is also included in the final cart & checkout figure along
         * with the results from GetQuote (below) */
	function get_item_shipping(&$cart_item) {
		global $wpdb;
		// If we're calculating a price based on a product, and that the store has shipping enabled
		$product_id = $cart_item->product_id;
		$quantity = $cart_item->quantity;
		$weight = $cart_item->weight;
		$unit_price = $cart_item->unit_price;
    		if (is_numeric($product_id) && (get_option('do_not_use_shipping') != 1)) {
			$country_code = $_SESSION['wpsc_delivery_country'];
			// Get product information
      			$product_list = $wpdb->get_row("SELECT *
			                                  FROM `".WPSC_TABLE_PRODUCT_LIST."`
				                         WHERE `id`='{$product_id}'
			                                 LIMIT 1",ARRAY_A);
       			// If the item has shipping enabled
      			if($product_list['no_shipping'] == 0) {
        			if($country_code == get_option('base_country')) {
					// Pick up the price from "Local Shipping Fee" on the product form
          				$additional_shipping = $product_list['pnp'];
				} else {
					// Pick up the price from "International Shipping Fee" on the product form
          				$additional_shipping = $product_list['international_pnp'];
				}          
        			$shipping = $quantity * $additional_shipping;
			} else {
        			//if the item does not have shipping
        			$shipping = 0;
			}
		} else {
      			//if the item is invalid or store is set not to use shipping
			$shipping = 0;
		}
    		return $shipping;	
	}
	
	/* This function returns an Array of possible shipping choices, and associated costs.
         * This is for the cart in general, per item charges (As returned from get_item_shipping (above))
         * will be added on as well. */
	function getQuote() {
		global $wpdb, $wpsc_cart, $wp_version;
		if ($this->base_country != 'NO' || strlen($this->base_zipcode) != 4 || !count($wpsc_cart->cart_items)) return;
		
		// This code is let here to show how you can access delivery address info
		// We don't use it for this skeleton shipping method
		$destzipcode = '';
		if(isset($_POST['zipcode'])) {
			$destzipcode = $_POST['zipcode'];      
			$_SESSION['wpsc_zipcode'] = $_POST['zipcode'];
		} else if(isset($_SESSION['wpsc_zipcode'])) {
			$destzipcode = $_SESSION['wpsc_zipcode'];
		}

		if (isset($_POST['country'])) {
			$country = $_POST['country'];
			$_SESSION['wpsc_delivery_country'] = $country;
		} else {
			$country = $_SESSION['wpsc_delivery_country'];
		}
		if($country != 'NO') {
			return array();
		}
		if(strlen($destzipcode) != 4) {
			// No destination postcode entered yet, so just return an empty set of quotes
			return array();
		}
		$options = get_option($this->getInternalName().'_options');
		$prod = "";
		$vol = "";
		foreach ( $this->services as $code => $value ) {
			if ( $options['services'][$code] == 1) {
				$prod .= '&product='.$code;
			}
		}
		if ( $options['use_cod'] == 1 ) {
			$prod.= '&additional=postoppkrav';
		}
		
		//Calculate the total cart dimensions by adding the volume of each product then calculating the cubed root
		$volume = 0;
		$volumeitemtotal = 0;
		$volumetotal = 0;
		$packagenum = 0;
		$weight = "";

		//foreach((array)$wpsc_cart->cart_items as $cart_item) {
		foreach($wpsc_cart->cart_items as $key => $cart_item) {
			$product_meta = get_post_meta($cart_item->product_id,'_wpsc_product_metadata');
			$meta = $product_meta[0]['dimensions'];
			if ($meta && is_array($meta)) {
				$productVolume = 1;
				foreach (array('width','height','length') as $dimension) {
					switch ($meta["{$dimension}_unit"]) {
						// we need the units in cm
						case 'cm':
							// keep cm
							$productVolume = $productVolume * (floatval($meta[$dimension]));
							break;
						case 'meter':
							// convert from m to cm
							$productVolume = $productVolume * (floatval($meta[$dimension]) * 100);
							break;
						case 'in':
							// convert from in to cm
							$productVolume = $productVolume * (floatval($meta[$dimension]) * 2.54);
							break;
					}
				}
				$volume += floatval($productVolume);
			}
			if ( $options['quote_method'] == 'items' ) {
				$volume = $volume / 1000;
				for ( $i = 0; $i < $cart_item->quantity; $i++ ) {
					$vol .= "&volume{$packagenum}={$volume}";
					// convert weight from pound to gram
					$gram = $cart_item->weight * 453.59237;
					$weight .= "&weightInGrams{$packagenum}={$gram}";
					$packagenum++;
				}
			} else {
				// convert volume from cubic cm to cubic dm
				$volumeitemtotal = $volume * $cart_item->quantity / 1000;
				$volumetotal = $volumetotal + $volumeitemtotal;
			}
		}
		
		if ( $options['quote_method'] == 'total' ) {
			$totalweight = floatval(wpsc_cart_weight_total() * 453.59237);
			$weight = "&weightInGrams={$totalweight}";
			if ( $volumetotal > 0 ) {
				$vol = "&volume={$volumetotal}";
			}
		}
		
		$validate_url = "http://fraktguide.bring.no/fraktguide/postalCode.xml?pnr={$destzipcode}";
		$base_url = "http://fraktguide.bring.no/fraktguide/products/price.xml?from={$this->base_zipcode}&to={$destzipcode}";
		$bring_url = $base_url.$prod.$weight.$vol;

		/*
		all.xml?weightInGrams=1500&from=7600&to=1407
		&length=30&width=40&height=40&volume=33&date=2009-2-3
		*/
		$header_options = array(
			'timeout' => 3,
			'user-agent' => 'WordPress/' . $wp_version . '; WP e-Commerce/' . WPSC_PRESENTABLE_VERSION . '; ' . home_url( '/' )
		);
		$results = wp_remote_get($validate_url, $header_options);
		$body = trim( $results['body'] );
		$xml = new SimpleXMLElement($body);
		$valid = $xml->xpath('/PostalCodeQueryResponse/Response/@valid');
		If ($valid[0] == "false") {
			// Invalid destination postcode entered, so just return an empty set of quotes
		    return array();
		}
		
		$response = wp_remote_get($bring_url, $header_options);
		
		if ( is_wp_error( $response ) ) {
			$error_string = $response->get_error_message();
			return false;
		}

		if ( 200 != $response['response']['code'] ) {
			return false;
		}
			
		$body = trim( $response['body'] );
		$xml = new SimpleXMLElement($body);
		$pricearray = array();
		foreach ($xml->Product as $product) {
			$prodID = (string)$product->ProductId;
			if ( $options['price_vat'] == 'with' ) {
				// Get price with VAT
				$price 	= floatval($product->Price->PackagePriceWithoutAdditionalServices->AmountWithVAT);
			} else {
				// Get price without VAT
				$price 	= floatval($product->Price->PackagePriceWithoutAdditionalServices->AmountWithoutVAT);
			}
			if ( $options['use_cod'] == 1) {
				// Get price for Collect on Delivery
				if ( $options['price_vat'] == 'with' ) {
					// Get price for CoD with VAT
					$cod = floatval($product->Price->AdditionalServicePrices->AdditionalService->AdditionalServicePrice->AmountWithVAT);
				} else {
					// Get price for CoD without VAT
					$cod = floatval($product->Price->AdditionalServicePrices->AdditionalService->AdditionalServicePrice->AmountWithoutVAT);
				}
				$totalprice = $price + $cod;
			} else {
				$totalprice = $price;
			}
			$pricearray[$prodID] = $totalprice;
		}
		asort($pricearray);
			  
		// Return an array of options for the user to choose
		// The first option is the default		
		foreach($pricearray as $key => $val) {
			$ProductName = $this->services[$key];
			$shippingArray[strval($ProductName)] = floatval($val);
		}
		
		if($country == 'NO') {
			return $shippingArray;
		} else {
			return;
		}
			  
	}
	
} 

function add_my_script() {
	wp_enqueue_script('my-script', plugins_url('js/bfg.js', __FILE__), array('jquery'), '0.3', true);
}

function bring_fraktguide_add($wpsc_shipping_modules) {
	global $bring_fraktguide;
	$bring_fraktguide = new bring_fraktguide();
	$wpsc_shipping_modules[$bring_fraktguide->getInternalName()] = $bring_fraktguide;
	return $wpsc_shipping_modules;
}

register_activation_hook( __FILE__, array('bring_fraktguide', 'bring_fraktguide_install') );
load_plugin_textdomain('bfg', false, dirname( plugin_basename(__FILE__) ) . '/lang');
add_filter('wpsc_shipping_modules', 'bring_fraktguide_add');
add_action('template_redirect', 'add_my_script');

?>